@@ -97,6 +97,7 @@ static void terminal_screen_size_request (GtkWidget *widget,
 static void terminal_screen_map         (GtkWidget           *widget);
 static void terminal_screen_update_on_realize (GtkWidget      *widget,
                                                TerminalScreen *screen);
+static void terminal_screen_change_font (TerminalScreen *screen);
 static gboolean terminal_screen_popup_menu         (GtkWidget      *term,
                                                     TerminalScreen *screen);
 static gboolean terminal_screen_button_press_event (GtkWidget      *term,
@@ -119,8 +120,6 @@ static void terminal_screen_widget_encoding_changed  (GtkWidget      *term,
 
 static void terminal_screen_setup_dnd                (TerminalScreen *screen);
 
-static void reread_profile (TerminalScreen *screen);
-
 static gboolean cook_title  (TerminalScreen *screen, const char *raw_title, char **old_cooked_title);
 
 static void terminal_screen_cook_title      (TerminalScreen *screen);
@@ -181,7 +180,7 @@ style_set_callback (GtkWidget *widget,
                     void      *data)
 {
   if (GTK_WIDGET_REALIZED (widget))
-    terminal_screen_update_on_realize (widget, TERMINAL_SCREEN (data));  
+    terminal_screen_change_font (TERMINAL_SCREEN (data));
 }
 
 static void
@@ -521,12 +520,13 @@ terminal_screen_get_icon_title_set (TerminalScreen *screen)
   return screen->priv->icon_title_set;
 }
 
-static void
-reread_profile (TerminalScreen *screen)
+void
+terminal_screen_reread_profile (TerminalScreen *screen)
 {
   TerminalProfile *profile;
   GtkWidget *term;
   TerminalBackgroundType bg_type;
+  TerminalWindow *window;
   
   profile = screen->priv->profile;  
   
@@ -549,7 +549,7 @@ reread_profile (TerminalScreen *screen)
     }
   
   if (GTK_WIDGET_REALIZED (screen->priv->term))
-    terminal_screen_update_on_realize (term, screen);
+    terminal_screen_change_font (screen);
 
   terminal_widget_set_cursor_blinks (term,
                                      terminal_profile_get_cursor_blink (profile));
@@ -600,13 +600,24 @@ reread_profile (TerminalScreen *screen)
 
   if (bg_type == TERMINAL_BACKGROUND_IMAGE ||
       bg_type == TERMINAL_BACKGROUND_TRANSPARENT)
+    {
     terminal_widget_set_background_darkness (term,
                                              terminal_profile_get_background_darkness (profile));
+      terminal_widget_set_background_opacity (term,
+					      terminal_profile_get_background_darkness (profile));
+    }      
   else
+    {
     terminal_widget_set_background_darkness (term, 0.0); /* normal color */
+      terminal_widget_set_background_opacity (term, 1);
+    }
   
+  window = terminal_screen_get_window (screen);
+  if (window == NULL || !terminal_window_uses_argb_visual (window))
   terminal_widget_set_background_transparent (term,
                                               bg_type == TERMINAL_BACKGROUND_TRANSPARENT);
+  else
+    terminal_widget_set_background_transparent (term, FALSE);
 
   terminal_widget_set_backspace_binding (term,
                                          terminal_profile_get_backspace_binding (profile));
@@ -698,7 +709,7 @@ profile_changed_callback (TerminalProfile           *profile,
                           const TerminalSettingMask *mask,
                           TerminalScreen            *screen)
 {
-  reread_profile (screen);
+  terminal_screen_reread_profile (screen);
 }
 
 static void
@@ -743,7 +754,7 @@ monospace_font_change_notify (GConfClient *client,
   
   if (strcmp (entry->key, MONOSPACE_FONT_KEY) == 0 &&
       GTK_WIDGET_REALIZED (widget))
-    terminal_screen_update_on_realize (widget, screen);
+    terminal_screen_change_font (screen);
 }
 
 PangoFontDescription *
@@ -768,16 +779,15 @@ get_system_monospace_font (void)
   return desc;
 }
 
-static void
-terminal_screen_update_on_realize (GtkWidget      *term,
-                                   TerminalScreen *screen)
+void
+terminal_screen_set_font (TerminalScreen *screen)
 {
   TerminalProfile *profile;
+  GtkWidget *term;
   
+  term = screen->priv->term;
   profile = screen->priv->profile;
   
-  update_color_scheme (screen);
-
   if (terminal_widget_supports_pango_fonts ())
     {
       PangoFontDescription *desc;
@@ -875,12 +885,30 @@ terminal_screen_update_on_realize (GtkWidget      *term,
          }
     }
   
+}
+
+static void
+terminal_screen_update_on_realize (GtkWidget      *term,
+                                   TerminalScreen *screen)
+{
+  TerminalProfile *profile;
+
+  profile = screen->priv->profile;
+
+  update_color_scheme (screen);
+  
   terminal_widget_set_allow_bold (term,
                                   terminal_profile_get_allow_bold (profile));
-
   terminal_window_set_size (screen->priv->window, screen, TRUE);
 }
 
+static void
+terminal_screen_change_font (TerminalScreen *screen)
+{
+  terminal_screen_set_font (screen);
+  terminal_screen_update_on_realize (screen->priv->term, screen);
+}
+
 static void
 profile_forgotten_callback (TerminalProfile *profile,
                             TerminalScreen  *screen)
@@ -946,7 +974,7 @@ terminal_screen_set_profile (TerminalScreen *screen,
 
   screen->priv->profile = profile;
 
-  reread_profile (screen);
+  terminal_screen_reread_profile (screen);
 
   if (screen->priv->profile)
     g_signal_emit (G_OBJECT (screen), signals[PROFILE_SET], 0);
@@ -2023,8 +2051,7 @@ terminal_screen_set_font_scale (TerminalScreen *screen,
       GTK_WIDGET_REALIZED (screen->priv->term))
     {
       /* Update the font */
-      terminal_screen_update_on_realize (screen->priv->term,
-                                         screen);
+      terminal_screen_change_font (screen);
     }
 }
 
@@ -2405,22 +2432,26 @@ drag_data_received  (GtkWidget *widget, GdkDragContext *context,
 	uris = g_strsplit (uri_list, "\r\n", 0);
 
         i = 0;
-        while (uris && uris[i])
+	while (uris && uris[i])
           {
             char *old;
             
             old = uris[i];
-
+	    /* First, treat the dropped URI like it's a filename */
             uris[i] = g_filename_from_uri (old, NULL, NULL);
-
-            /* If the URI wasn't a filename, then pass it through
-             * as a URI, so you can DND from Mozilla or whatever
-             */
-            if (uris[i] == NULL)
-              uris[i] = old;
-            else
-              g_free (old);
-            
+	    /* if it's NULL, that means it wasn't a filename.
+	     * Pass it as a plain URI, then.
+	     */
+	    if (uris[i] == NULL)
+	      uris[i] = old;
+            else 
+	      {
+		/* OK, it's a file. Quote the shell characters. */
+		g_free(old);
+		old = uris[i];
+		uris[i] = g_shell_quote(uris[i]);
+		g_free(old);
+	      }
             ++i;
           }
 
