@@ -85,6 +85,9 @@ struct _TerminalWindowPrivate
    * event-driven while GTK doesn't support _NET_WM_STATE_FULLSCREEN
    */
   guint fullscreen : 1;
+
+  /* Compositing manager integration */
+  guint have_argb_visual : 1;
 };
 
 enum {
@@ -749,6 +752,34 @@ edit_menu_activate_callback (GtkMenuItem *menuitem,
   gtk_clipboard_request_text (window->priv->clipboard, (GtkClipboardTextReceivedFunc) update_edit_menu, window);
 }
 
+static void
+initialize_alpha_mode (TerminalWindow *window)
+{
+  GdkScreen *screen;
+  GdkColormap *colormap;
+
+  screen = gtk_widget_get_screen (GTK_WIDGET (window));
+  colormap = gdk_screen_get_rgba_colormap (screen);
+  if (colormap != NULL && gdk_screen_is_composited (screen))
+    {
+      /* Set RGBA colormap if possible so VTE can use real alpha
+       * channels for transparency. */
+
+      gtk_widget_set_colormap(GTK_WIDGET (window), colormap);
+      window->priv->have_argb_visual = TRUE;
+    }
+  else
+    {
+      window->priv->have_argb_visual = FALSE;
+    }
+}
+
+gboolean
+terminal_window_uses_argb_visual (TerminalWindow *window)
+{
+  return window->priv->have_argb_visual;
+}
+
 static void
 terminal_window_init (TerminalWindow *window)
 {
@@ -781,6 +812,8 @@ terminal_window_init (TerminalWindow *window)
   window->priv->use_mnemonics = TRUE;
   window->priv->using_mnemonics = FALSE;
 
+  initialize_alpha_mode (window);
+
   /* force gtk to construct its GtkClipboard; otherwise our UI is very slow the first time we need it */
   window->priv->clipboard = gtk_clipboard_get_for_display (gtk_widget_get_display (GTK_WIDGET (window)), GDK_NONE);
 
@@ -1536,14 +1569,6 @@ terminal_window_set_size_force_grid (TerminalWindow *window,
   app = gtk_widget_get_toplevel (widget);
   g_assert (app != NULL);
 
-  /* This set_size_request hack is because the extra size above base
-   * size should only include the width of widgets that intersect the
-   * term vertically and the height of widgets that intersect the term
-   * horizontally. It works around a GTK bug, GTK should handle
-   * this case. The size request can be huge without hosing
-   * anything because we set the MIN_SIZE geometry hint.
-   */
-  gtk_widget_set_size_request (widget, 2000, 2000);
   gtk_widget_size_request (app, &toplevel_request);
   gtk_widget_size_request (widget, &widget_request);
 
@@ -1803,6 +1828,10 @@ notebook_tab_detached_callback (GtkWidget       *notebook,
                                 TerminalScreen  *screen,
                                 TerminalWindow  *window)
 {
+  /* Reset the tab menu before detaching the tab, in case the tab changed
+   * position before it was detached (bug #330246). */
+  reset_tab_menuitems (window);
+
   detach_tab (screen, window);
 }
 
@@ -1811,6 +1840,7 @@ notebook_tab_removed_callback (GtkWidget       *notebook,
                                TerminalScreen  *screen,
                                TerminalWindow  *window)
 {  
+  gboolean single;
   /* Called from terminal_notebook_move_tab() */
   if (find_screen (window, screen) != NULL) {
     g_assert (terminal_screen_get_window (screen) == window);
@@ -1842,6 +1872,11 @@ notebook_tab_removed_callback (GtkWidget       *notebook,
   reset_tab_menuitems (window);
   update_tab_sensitivity (window);
 
+  /* The tab bar may have disappeared */
+  single = g_list_length (window->priv->terms) == 1;
+  if (single)
+    terminal_window_set_size (window, window->priv->active_term, TRUE);
+
   /* Close window if no more terminals */
   if (window->priv->terms == NULL)
     gtk_widget_destroy (GTK_WIDGET (window));
@@ -2250,7 +2285,6 @@ confirm_close_window (TerminalWindow *window)
   GtkWidget *dialog;
   GError *error;
   gboolean result;
-  char *msg, *msg1;
   int n;
 
   n = gtk_notebook_get_n_pages (GTK_NOTEBOOK (window->priv->notebook));
@@ -2262,32 +2296,26 @@ confirm_close_window (TerminalWindow *window)
   if (!gconf_client_get_bool (window->priv->conf, CONF_GLOBAL_PREFIX "/confirm_window_close", &error))
     return TRUE;
 
-  msg1 = g_strdup_printf (ngettext ("This window has one tab open. Closing "
-				    "the window will close it.",
-				    "This window has %d tabs open. Closing "
-				    "the window will also close all of its "
-				    "tabs.",
-				    n),
-			  n);
-
-  msg = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n"
-			 "%s\n", _("Close all tabs?"), msg1);
-
-  dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (window),
+  dialog = gtk_message_dialog_new (GTK_WINDOW (window),
                                    GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_WARNING,
-                                   GTK_BUTTONS_NONE,
-                                   msg);
+                                   GTK_BUTTONS_CANCEL,
+                                   "%s", _("Close all tabs?"));
+  gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+			  		    ngettext ("This window has one tab open. Closing "
+						      "the window will close it.",
+						      "This window has %d tabs open. Closing "
+						      "the window will also close all of its "
+						      "tabs.",
+						      n),
+					    n);
+
   gtk_window_set_title (GTK_WINDOW(dialog), ""); 
-  g_free (msg);
-  g_free (msg1);
 
-  gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-  gtk_dialog_add_button (GTK_DIALOG (dialog), _("Close All _Tabs"), GTK_RESPONSE_YES);
+  gtk_dialog_add_button (GTK_DIALOG (dialog), _("Close All _Tabs"), GTK_RESPONSE_ACCEPT);
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
 
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
-
-  result = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES;
+  result = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT;
   gtk_widget_destroy (dialog);
 
   return result;
@@ -2775,9 +2803,6 @@ detach_tab_callback(GtkWidget      *menuitem,
   page = gtk_notebook_get_nth_page (notebook, page_num);
   
   detach_tab (TERMINAL_SCREEN (page), window);
-  
-  update_tab_sensitivity (window);
-  reset_tab_menuitems (window);
 }
 
 static void
@@ -2808,8 +2833,6 @@ static void
 about_callback (GtkWidget      *menuitem,
                 TerminalWindow *window)
 {
-  static GtkWidget *about = NULL;
-
   const char *copyright =
     "Copyright \xc2\xa9 2002-2004 Havoc Pennington\n"
     "Copyright \xc2\xa9 2003-2004 Mariano Su\303\241rez-Alvarez\n"
@@ -2821,50 +2844,34 @@ about_callback (GtkWidget      *menuitem,
     NULL
   };
   const gchar *license[] = {
-    "GNOME Terminal is free software; you can redistribute it and/or modify \n"
-    "it under the terms of the GNU General Public License as published by \n"
-    "the Free Software Foundation; either version 2 of the License, or \n"
+    "GNOME Terminal is free software; you can redistribute it and/or modify "
+    "it under the terms of the GNU General Public License as published by "
+    "the Free Software Foundation; either version 2 of the License, or "
     "(at your option) any later version.",
-    "GNOME Terminal is distributed in the hope that it will be useful, \n"
-    "but WITHOUT ANY WARRANTY; without even the implied warranty of \n"
-    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n"
+    "GNOME Terminal is distributed in the hope that it will be useful, "
+    "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
     "GNU General Public License for more details.",
-    "You should have received a copy of the GNU General Public License \n"
-    "along with Nautilus; if not, write to the Free Software Foundation, Inc., \n"
+    "You should have received a copy of the GNU General Public License "
+    "along with Nautilus; if not, write to the Free Software Foundation, Inc., "
     "51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA"
   };
-  const gchar *license_text;
+  gchar *license_text;
 
-  if (about)
-    {
-      gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (window));
-      gtk_window_present (GTK_WINDOW (about));
-      return;
-    }
-				     
-  about = gtk_about_dialog_new ();
+  license_text = g_strjoin ("\n\n",
+			    _(license[0]), _(license[1]), _(license[2]), NULL);
 
-  license_text = g_strconcat (license[0], "\n\n", license[1], "\n\n",
-			      license[2], "\n\n", NULL);
-
-  gtk_about_dialog_set_license (GTK_ABOUT_DIALOG (about), license_text);
-
-  gtk_about_dialog_set_name (GTK_ABOUT_DIALOG (about), _("GNOME Terminal"));
-  gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG(about), _("A terminal emulator for the GNOME desktop"));
-  gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (about), VERSION);
-  gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (about), copyright);
-  gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG (about), authors);
-  gtk_about_dialog_set_translator_credits (GTK_ABOUT_DIALOG (about), _("translator-credits"));
-  gtk_about_dialog_set_logo_icon_name (GTK_ABOUT_DIALOG (about), "gnome-terminal");
-
-  gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (window));
-  gtk_window_set_destroy_with_parent (GTK_WINDOW (about), TRUE);
-
-  terminal_util_set_unique_role (GTK_WINDOW (about), "gnome-terminal-about");
-  
-  g_object_add_weak_pointer (G_OBJECT (about), (void**) &about);
-
-  gtk_widget_show (about);
+  gtk_show_about_dialog (GTK_WINDOW (window),
+			 "name", _("GNOME Terminal"),
+			 "copyright", copyright,
+			 "comments", _("A terminal emulator for the GNOME desktop"),
+			 "version", VERSION,
+			 "authors", authors,
+			 "license", license_text,
+			 "wrap-license", TRUE,
+			 "translator-credits", _("translator-credits"),
+			 "logo-icon-name", "gnome-terminal",
+			 NULL);
 }
 
 
